(*****************************************************************************)
(** COPYRIGHT (c) 2002 Niclas Svan (niclas.svan@home.se)                     *)
(*****************************************************************************)
(** Description:                                                             *)
(** A simple class that provides the basic functions for a hash table.       *)
(** These types of tables are also called associative arrays                *)
(** Note! This class only handles strings                                    *)
(**                                                                          *)
(** Version 1.2                                                              *)
(*****************************************************************************)

unit uNSStringHashTable;

interface
    
uses
  SysUtils, Classes, IniFiles;

Type
  TNSStringHashTable = class(TObject)

  public
    constructor Create();
    destructor Destroy();override;
    function Add(const strKey : string; const strValue : string) : boolean;
    function AllValuesEquals(const strValue : string) : boolean;
    function Count():integer;
    function Delete(const strKey : string) : boolean;
    procedure DeleteAll();
    function GetValue(const strKey : string): string;
    function GetAllValues() : TStringList;
    function GetAllKeys() : TStringList;
    function GetAllKeysAndValues : TStringList;
    function GetAllKeysForValue(const strValue : string) : TStringList;
    function KeyExists(const strKey : string) : boolean;
    function SetValue(const strKey :string; const  strValue : string) : boolean;
    procedure SetAllValuesTo(const strValue : string);
    function ValueExists(const strValue : string) : boolean;
  private
    arrKeys : THashedStringList;
    arrValues : THashedStringList;
End;

implementation

{ TNSStringHashTable }

constructor TNSStringHashTable.Create();
begin
  inherited Create;
  arrKeys := THashedStringList.Create;
  arrValues := THashedStringList.Create;
end;

function TNSStringHashTable.Add(const strKey: string; const strValue: string) : boolean;
begin
    if (Length(strKey)< 1) then
    begin
      Result := FALSE;
      Exit;
    end;
    if arrKeys.IndexOf(strKey) < 0 then
    begin
      arrKeys.Add(strKey);
      arrValues.Add(strValue);
      Result := TRUE;
    end
    else
      Result := FALSE;
end;


function TNSStringHashTable.Count: integer;
begin
  Result := arrKeys.Count;
end;

function TNSStringHashTable.GetValue(const strKey: string): string;
var index :integer;
begin
  index := arrKeys.IndexOf(strKey);
  if index > -1 then
    Result := arrValues.Strings[index]
  else
    Result := '';
end;

function TNSStringHashTable.Delete(const strKey : string): boolean;
var index :integer;
begin
  index := arrKeys.IndexOf(strKey);
  if index > -1 then
  begin
    arrKeys.Delete(index);
    arrValues.Delete(index);
    Result := TRUE;
  end
  else
    Result := FALSE;
end;


function TNSStringHashTable.SetValue(const strKey : string; const strValue: string): boolean;
var index :integer;
begin
  index := arrKeys.IndexOf(strKey);
  if index > -1 then
  begin
    arrValues.Strings[index] := strValue;
    Result := TRUE;
  end
  else
    Result := FALSE;
end;

function TNSStringHashTable.GetAllValues: TStringList;
var
  index : integer;
  tmpSlist : TStringList;
begin
  tmpSlist :=  TStringList.Create;
  for index := 0 to arrValues.Count -1 do
    tmpSlist.Add(arrValues.Strings[index]);
  Result := tmpSlist;
end;

function TNSStringHashTable.GetAllKeys: TStringList;
var
  index : integer;
  tmpSlist : TStringList;
begin
  tmpSlist :=  TStringList.Create;
  for index := 0 to arrKeys.Count -1 do
    tmpSlist.Add(arrKeys.Strings[index]);
  Result := tmpSlist;

end;

function TNSStringHashTable.GetAllKeysAndValues: TStringList;
var
  index : integer;
  tmpSlist : TStringList;
begin
  tmpSlist :=  TStringList.Create;
  for index := 0 to arrKeys.Count -1 do
  begin
    tmpSlist.Add(arrKeys.Strings[index]);
    tmpSlist.Add(arrValues.Strings[index]);
  end;
  Result := tmpSlist;

end;

destructor TNSStringHashTable.Destroy;
begin
  arrKeys.Free;
  arrValues.Free;
end;

procedure TNSStringHashTable.SetAllValuesTo(const strValue: string);
var index : integer;
begin
  for index := 0 to arrValues.Count - 1 do
    arrValues.Strings[index] := strValue;
end;

function TNSStringHashTable.AllValuesEquals(const strValue: string): boolean;
var
  index : integer;
  bIsEqual : boolean;
begin
  bIsEqual := TRUE;
  for index := 0 to arrValues.Count - 1 do
  begin
    if (arrValues.Strings[index] <> strValue) then
    begin
      bIsEqual := FALSE;
      break;
    end;
  end;
  Result := bIsEqual;

end;

procedure TNSStringHashTable.DeleteAll;
var index : integer;
begin
  for index := 0 to arrValues.Count - 1 do
  begin
    arrKeys.Delete(0);
    arrValues.Delete(0);
  end;
end;

function TNSStringHashTable.KeyExists(const strKey: string): boolean;
begin
  if arrKeys.IndexOf(strKey) < 0 then
    Result := FALSE
  else
    Result := TRUE;
end;

function TNSStringHashTable.ValueExists(const strValue: string): boolean;
begin
  if arrValues.IndexOf(strValue) < 0 then
    Result := FALSE
  else
    Result := TRUE;
end;

function TNSStringHashTable.GetAllKeysForValue(
  const strValue: string): TStringList;
var
  index : integer;
  tmpSlist : TStringList;
begin
  tmpSlist :=  TStringList.Create;
  for index := 0 to arrValues.Count - 1 do
  begin
    if (arrValues.Strings[index] = strValue) then
      tmpSlist.Add(arrKeys.Strings[index]);
  end;
  Result := tmpSlist;
end;

end.
